#include "graphics.hpp"

// Variables
Uint32 last_time;
Uint32 current_time;
float frame_time;
float fps;
float frame_time_graph[100];

SDL_GLContext gl_context;

// Functions
int initialize_opengl() {
    // Initialize OpenGL
    SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, open_gl_major_version);
    SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, open_gl_minor_version);
    SDL_GL_SetAttribute(SDL_GL_CONTEXT_PROFILE_MASK, SDL_GL_CONTEXT_PROFILE_CORE);

    // Initialize OpenGL context
    gl_context = SDL_GL_CreateContext(window);
    if (!gl_context) {
        std::cerr << "OpenGL context could not be created! SDL_Error: " << SDL_GetError() << std::endl;
        return 1; // OpenGL context creation failed
    }

    // Initialize OpenGL settings for the window
    SDL_GL_MakeCurrent(window, gl_context);
    SDL_GL_SetSwapInterval(enable_vsync);
    SDL_SetWindowPosition(window, SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED);
    SDL_ShowWindow(window);

    glViewport(0, 0, (int)(window_width * main_scale), (int)(window_height * main_scale));

    return 0; // OpenGL initialization successful
}

void cleanup_opengl() {
    // Destroy OpenGL context
    if (gl_context) {
        SDL_GL_MakeCurrent(NULL, NULL);
        SDL_GL_DestroyContext(gl_context);
        gl_context = nullptr;
    }
    std::cout << "OpenGL cleanup successful." << std::endl;
}

unsigned int compile_shader(const char* shader_source, GLenum shader_type) {
    unsigned int shader = glCreateShader(shader_type);
    if (shader == 0) {
        std::cerr << "Failed to create shader." << std::endl;
        return 0; // Shader creation failed
    }

    glShaderSource(shader, 1, &shader_source, nullptr);
    glCompileShader(shader);

    // Check for compilation errors
    int success;
    glGetShaderiv(shader, GL_COMPILE_STATUS, &success);
    if (!success) {
        char info_log[512];
        glGetShaderInfoLog(shader, sizeof(info_log), nullptr, info_log);
        std::cerr << "Shader compilation error: " << info_log << std::endl;
        glDeleteShader(shader);
        return 0; // Shader compilation failed
    }

    return shader; // Shader compiled successfully
}

unsigned int create_program(unsigned int vertex_shader, unsigned int fragment_shader) {
    unsigned int program = glCreateProgram();
    if (program == 0) {
        std::cerr << "Failed to create shader program." << std::endl;
        return 0; // Program creation failed
    }

    glAttachShader(program, vertex_shader);
    glAttachShader(program, fragment_shader);
    glLinkProgram(program);

    // Check for linking errors
    int success;
    glGetProgramiv(program, GL_LINK_STATUS, &success);
    if (!success) {
        char info_log[512];
        glGetProgramInfoLog(program, sizeof(info_log), nullptr, info_log);
        std::cerr << "Shader program linking error: " << info_log << std::endl;
        glDeleteProgram(program);
        return 0; // Program linking failed
    }

    return program; // Program created successfully
}

int initialize_imgui() {
    IMGUI_CHECKVERSION();
    ImGui::CreateContext();
    ImGuiIO& io = ImGui::GetIO(); (void)io;
    io.ConfigFlags |= ImGuiConfigFlags_NavEnableKeyboard;
    io.ConfigFlags |= ImGuiConfigFlags_DockingEnable;

    ImGui::StyleColorsDark();
    ImGuiStyle& style = ImGui::GetStyle();
    style.ScaleAllSizes(main_scale);
    style.FontScaleDpi = main_scale;

    if (!ImGui_ImplSDL3_InitForOpenGL(window, gl_context)) {
        std::cerr << "ImGui_ImplSDL3_InitForOpenGL failed!" << std::endl;
        return 1; // Initialization failed
    }

    char glsl_version[32];
    snprintf(glsl_version, sizeof(glsl_version), "#version %d%d0", open_gl_major_version, open_gl_minor_version);
    if (!ImGui_ImplOpenGL3_Init(glsl_version)) {
        std::cerr << "ImGui_ImplOpenGL3_Init failed!" << std::endl;
        ImGui_ImplSDL3_Shutdown();
        ImGui::DestroyContext();
        return 1; // Initialization failed
    }

    return 0;
}

void cleanup_imgui() {
    ImGui_ImplOpenGL3_Shutdown();
    ImGui_ImplSDL3_Shutdown();
    ImGui::DestroyContext();
    std::cout << "ImGui cleanup successful." << std::endl;
}

void calc_frame_time() {
    // Calculate frame time
    current_time = SDL_GetTicks();
    frame_time = (current_time - last_time) / 1000.0f; // Convert milliseconds to seconds
    fps = 1.0f / frame_time;
    last_time = current_time;

    // Update frame time graph
    for (int i = 99; i > 0; --i) {
        frame_time_graph[i] = frame_time_graph[i - 1]; // Shift old values
    }
    frame_time_graph[0] = frame_time; // Add new frame time
}

void new_frame_imgui() {
    ImGui_ImplOpenGL3_NewFrame();
    ImGui_ImplSDL3_NewFrame();
    ImGui::NewFrame();
}

void frame_time_ui() {
    ImGui::Begin("Frame Time");
    ImGui::Text("Current FPS: %.2f", fps);
    ImGui::Text("Current Frame Time: %.3f ms", frame_time * 1000.0f);
    ImGui::PlotLines("##Frame Time Graph", frame_time_graph, 100, 0, "Frame Time (ms)", 0.0f, 0.1f, ImVec2(200, 80));
    ImGui::End();
}